/***************************************************************************
                          knmap.cpp  -  description
                             -------------------
    begin                : Thu May 25 2000
    copyright            : (C) 2000 by Alexandre Sagala
    email                : rewind2@videotron.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kiconloader.h>
#include <kmsgbox.h>

#include <qcolor.h>
#include <qstrlist.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>

#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>

#include "knmap.h"
#include "resource.h"
#include "wordwrap.h"

KNmap::KNmap()
{
	bRoot = false;
	bBounce = false;
	bInputFile = false;
	bRpc = false;
	bTcpScan = true;
	logId = ID_LOG_NORMAL;
	scanId = TCP_SCAN;

	initGUI();
	initMenuBar();
	initConnect();
	restoreConfig();
	rootCheck();
	
	toggleDecoys(false);	

}

KNmap::~KNmap()
{  	
	delete fileMenu;
	delete logMenu;
	delete optionMenu;
	delete helpMenu;
}

void KNmap::restoreConfig()
{
	QStrList hosts;
	
	kapp->getConfig()->readListEntry("hosts", hosts);
	
	
	QStrListIterator it(hosts);
 	for ( ; it.current(); ++it )
 	{
 	   char *host = it.current();
 		cboHost->insertItem(host);
 	}
	
}

void KNmap::initConnect()
{
  	///////////////////////////////////////////////////////////////////
  	// Connect SIGNALS & SLOTS

  	connect(groupScan, SIGNAL(clicked(int)), SLOT(slotGroupScan(int)));
  	connect(chkBounce, SIGNAL(clicked()), SLOT(slotBounce()));
  	connect(chkRpc, SIGNAL(clicked()), SLOT(slotRpc()));
  	connect(cmdAdd, SIGNAL(clicked()), SLOT(slotAdd()));
  	connect(cmdDel, SIGNAL(clicked()), SLOT(slotDel()));
  	connect(cmdScan, SIGNAL(clicked()), SLOT(slotScan()));
  	connect(cmdQuit, SIGNAL(clicked()), SLOT(slotQuit()));
  	connect(cmdCancel, SIGNAL(clicked()), SLOT(slotCancel()));
  	connect(cmdInput, SIGNAL(clicked()), SLOT(slotInput()));
  	  	  	
  	connect(&scan,SIGNAL(processExited(KProcess*)),this,SLOT(slotScanExited(KProcess*)));
  	connect(&scan,SIGNAL(receivedStdout(KProcess*,char*,int)),this,SLOT(slotScanStdout(KProcess*,char*,int)));
  	connect(&scan,SIGNAL(receivedStderr(KProcess*,char*,int)),this,SLOT(slotScanStderr(KProcess*,char*,int)));
}

void KNmap::toggleDecoys(bool value)
{
	cmdAdd->setEnabled(value);
	txtDecoy->setEnabled(value);
	lstDecoy->setEnabled(value);
	
	if (value == true)
		lstDecoy->setBackgroundColor(QColor("white"));
	else
		lstDecoy->setBackgroundColor(QColor("grey"));		
}

void KNmap::rootCheck()
{
	// if not root user
	if ( getuid() != 0)
	{
		disableNonRoot();
		bRoot = false;
		KMsgBox::message (this,"KNmap","You are not root. Some \noptions will be missing",KMsgBox::INFORMATION);
	}
	else
		bRoot = true;	
}

void KNmap::disableNonRoot()
{
	optSyn->setEnabled(false);
	optFin->setEnabled(false);
	optXmas->setEnabled(false);
	optNull->setEnabled(false);
	optUdp->setEnabled(false);
	
	chkOs->setEnabled(false);
	
	toggleDecoys(false);
	
	cboPing->removeItem (4);	// TCP & ICMP
	cboPing->removeItem (3);	// ICMP
	cboPing->setCurrentItem (1);	// TCP
}

void KNmap::toggleOthers(bool value)
{
   if (bRoot)
		chkOs->setEnabled(value);
		
   chkBounce->setEnabled(value);
	chkFastScan->setEnabled(value);
	chkRpc->setEnabled(value);
	chkRandom->setEnabled(value);
	txtPort->setEnabled(value);
}

void KNmap::toggleScan(bool value)
{
	cmdScan->setEnabled(value);
	cmdCancel->setEnabled(!value);
}

void KNmap::initMenuBar()
{
	/////////////////////////
	// File Menu
	
	fileMenu = new QPopupMenu();
  	fileMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open..."), ID_FILE_OPEN);
  	fileMenu->insertItem(Icon("filefloppy.xpm"), i18n("&Save..."), ID_FILE_SAVE);  	
  	fileMenu->insertSeparator();
  	fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);
  	
  	/////////////////////////
	// Log Menu
	
	logMenu = new QPopupMenu();
  	logMenu->insertItem(i18n("&Normal format"), ID_LOG_NORMAL);
  	logMenu->insertItem(i18n("&Human format"), ID_LOG_HUMAN);
  	logMenu->insertItem(i18n("&Parsable format"), ID_LOG_MACHINE);
  	logMenu->insertSeparator();
  	logMenu->insertItem(i18n("&Verbose"), ID_LOG_VERBOSE);
  	logMenu->setItemChecked (ID_LOG_NORMAL, true);

  	
  	/////////////////////////
	// Option Menu
	
	optionMenu = new QPopupMenu();
  	optionMenu->insertItem(i18n("&Options"), ID_OPTION_OPTION);
  	
  	/////////////////////////
  	// Help Menu

  	QString aboutstring=kapp->appName()+" "+VERSION+"\n\n"+"author: Alexandre Sagala\nemail:  rewind2@videotron.ca";

  	helpMenu = new QPopupMenu();
  	helpMenu = kapp->getHelpMenu(true, aboutstring);

  	/////////////////////////
	// Menubar
	
  	menuBar()->insertItem(i18n("&File"), fileMenu);
  	menuBar()->insertItem(i18n("&Log"), logMenu);
  	menuBar()->insertItem(i18n("&Options"), optionMenu);

  	menuBar()->insertSeparator();
  	menuBar()->insertItem(i18n("&Help"), helpMenu);

  	///////////////////////////////////////////////////////////////////
  	// Connect SIGNALS & SLOTS

  	connect(fileMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  	connect(logMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  	connect(optionMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));

}

void KNmap::commandCallback(int id_)
{
	switch (id_)
	{
		case ID_FILE_OPEN:
			slotFileOpen();
         break;			
 		case ID_FILE_SAVE:
 			slotFileSave();
         break;
 		case ID_FILE_QUIT:
 		   slotQuit();
         break;			
		case ID_LOG_HUMAN:	
 		case ID_LOG_NORMAL:
 		case ID_LOG_MACHINE:
		case ID_LOG_VERBOSE:
		   slotLog(id_);		
         break;
		case ID_OPTION_OPTION:
			slotOption();
         break;
    	default:
         break;
	}
}
void KNmap::uncheckLogMenu()
{
	logMenu->setItemChecked (ID_LOG_HUMAN, false);
	logMenu->setItemChecked (ID_LOG_NORMAL, false);
	logMenu->setItemChecked (ID_LOG_MACHINE, false);
}

void KNmap::slotLog(int id_)
{
	switch(id_)
	{
		case ID_LOG_HUMAN:	
 		case ID_LOG_NORMAL:
 		case ID_LOG_MACHINE:
 			logId = id_;
 		   uncheckLogMenu();
 			logMenu->setItemChecked (id_, !logMenu->isItemChecked (id_));
 			break;
		case ID_LOG_VERBOSE:
			logMenu->setItemChecked (id_, !logMenu->isItemChecked (id_));
			break;
		default:
			break;
	}
}

void KNmap::slotGroupScan(int id_)
{
	scanId = id_;

	if (id_ == SYN_SCAN || id_ == FIN_SCAN || id_ == NULL_SCAN || id_ == XMAS_SCAN)
		chkFragment->setEnabled(true);	
	else
		chkFragment->setEnabled(false);
	
	if (id_ == TCP_SCAN)
	{
		bTcpScan = true;
		toggleDecoys(false);
		chkIdentd->setEnabled(true);
	}
	else
	{
		bTcpScan = false;
		if (bRoot)
			if (!bBounce || !bRpc )
				toggleDecoys(true);
		chkIdentd->setEnabled(false);
	}
	
	if (id_ == PING_SCAN)
		toggleOthers(false);
	else
		toggleOthers(true);
}

void KNmap::slotRpc()
{
	bRpc = chkRpc->isOn();
	
	if (!bTcpScan )
	{
		if (bRpc)
			toggleDecoys(false);
		else if (bBounce)
			;	// do nothing
		else
			toggleDecoys(true);
	}	
}

void KNmap::slotBounce()
{	
	bBounce = chkBounce->isOn();
	
	if (!bTcpScan || bRpc)
	{
		if (bBounce)
			toggleDecoys(false);
		else if (bRpc)
			;	// do nothing
		else
			toggleDecoys(true);
	}		
}

void KNmap::slotAdd()
{
	QString sDecoy;
	
	sDecoy = txtDecoy->text();
	sDecoy = sDecoy.stripWhiteSpace();	
	
	if (!sDecoy.isEmpty())
	{
		lstDecoy->insertItem(sDecoy);
		txtDecoy->clear();
		if (!cmdDel->isEnabled())
			cmdDel->setEnabled(true);
	}
}

void KNmap::slotDel()
{
	if (lstDecoy->count() > 0)
		if (lstDecoy->currentItem() != -1)
			lstDecoy->removeItem(lstDecoy->currentItem());
	
	if (lstDecoy->count() == 0)
		cmdDel->setEnabled(false);
}

void KNmap::slotScanStdout(KProcess *proc,char *buffer, int buflen)
{
	// @TODO wordwrap
	
	QString out(buffer);
	
	out.truncate(buflen);
	out = out.stripWhiteSpace();
	
	if(!out.isEmpty())
		txtResult->append(out);	
		
	//WordWrap wrapped(out,txtResult->width());
	
	//txtResult->insertLine(wrapped.wrapString());

}

void KNmap::slotScanStderr(KProcess *proc,char *buffer, int buflen)
{
	// @TODO wordwrap
	QString err(buffer);
	
	txtResult->clear();
	err.truncate(buflen);
	err = err.stripWhiteSpace();
	
	if (err.contains ("QUITTING!")>0)
		return;
		
	if (err.contains ("Warning")>0)
		return;	
			
	if(!err.isEmpty())
		KMsgBox::message (this,"KNmap",err,KMsgBox::STOP);
	
}

void KNmap::slotScanExited(KProcess *proc)
{
	toggleScan(true);
	scan.clearArguments();
}

QString KNmap::getScanType()
{
	QString scanType;
	
	switch(scanId)
	{
		case TCP_SCAN:
			scanType = "-sT";
			break;	
 		case SYN_SCAN:
			scanType = "-sS";
			break;	
		case FIN_SCAN:
			scanType = "-sF";
			break;	
		case XMAS_SCAN:
			scanType = "-sX";
			break;	
		case NULL_SCAN:
			scanType = "-sN";
			break;
		case UDP_SCAN:
			scanType = "-sU";
			break;
		case PING_SCAN:
			scanType = "-sP";
			break;	
	}
	return scanType;
}

QString KNmap::getLogType()
{
	QString logType;
	
	switch(logId)
	{
		case ID_LOG_HUMAN:
			logType = "-oN -";
			break;					
		case ID_LOG_MACHINE:
			logType = "-oM -";
			break;		
	}	
	return logType;
}

QString KNmap::getPingType()
{
	QString pingType;
	
	switch(cboPing->currentItem())
	{
		case TCP_ICMP:
			pingType = "-PB";
			break;
		case TCP:
			pingType = "-PT";
			break;
		case ICMP:
			pingType = "-PI";
			break;
		case SYN:
			pingType = "-PS";
			break;
		case NO_PING:
			pingType = "-P0";
			break;
	}
	return pingType;
}

QStrList KNmap::getOtherOptions()
{
	QStrList otherOptions;	
	
	// fragment
	if(chkFragment->isChecked())
		otherOptions.append("-f");
	
	// fastscan
	if(chkFastScan->isChecked())
		otherOptions.append("-F");
	
	// OS check
	if(chkOs->isChecked())
		otherOptions.append("-O");
		
	// identd
	if(chkIdentd->isChecked())
		otherOptions.append("-I");
			
	// rpc
	if(chkRpc->isChecked())
		otherOptions.append("-sR");
	
	// No ramdom		
	if(chkRandom->isChecked())
		otherOptions.append("-r");	
	
	// timing
	QString timing(cboTiming->currentText());
	if(timing != "Normal")
	{
		timing = "-T " + timing;
		otherOptions.append(timing);			
	}			
	
	// resolve
	QString resolve(cboResolve->currentText());
	if(resolve == "Always")
		otherOptions.append("-R");				
	else if (resolve == "Never")
		otherOptions.append("-n");					
		
	// port
	QString portRange;
	portRange = txtPort->text();
	portRange = portRange.stripWhiteSpace();	
	
	if (!portRange.isEmpty())
	{
		portRange = "-p" + portRange;
		otherOptions.append(portRange);			
	}

	// verbose
	if(logMenu->isItemChecked (ID_LOG_VERBOSE))
		otherOptions.append("-v");			

	// decoys
	if(lstDecoy->count() > 0)
	{
		QString decoy("-D");
		for (uint i = 0; i < lstDecoy->count();i++ )
		{
			decoy += lstDecoy->text(i);
			if (i != lstDecoy->count() -1)
				decoy += ",";		
		}	
		otherOptions.append(decoy);	
	}
	
	// @TODO bounce scanning
	// bounce
	
	return otherOptions;
}	

QString KNmap::getFileName(QString host)
{
	QString fileName;
	
	if (host.contains("-")>0)
	{
		int pos = host.find("-");
		fileName = host.mid(pos+2,host.length());
	}
	
	return fileName;
}

void KNmap::slotScan()
{
	QString host,scanType,logType,pingType;
	QStrList otherOptions;
	
	// @TODO validate fields 10..2.3 fucks everything
	
	host = cboHost->currentText();
	txtResult->clear();
	
	// @TODO get value from option dialog	
	scan<<"nmap";

	scanType = getScanType();
	
	pingType = getPingType();	
	if(pingType != "-PB")
		scan<<pingType;
	
	if(logId != ID_LOG_NORMAL)
	{		
		logType = getLogType();
		scan<<logType;
	}
	
 	scan<<scanType;	
 	
	otherOptions = getOtherOptions();	
	   	
 	QStrListIterator it(otherOptions); // iterator
 	for ( ; it.current(); ++it )
 	{
 	   char *option = it.current();
 		scan<<option;
 	}
 	
	if (host.contains("Input") > 0 || host.contains("File") > 0)
	{
	 	QString hostSwitch("-iL ");
		
		if(bInputFile)
		{
			hostSwitch += fileForHost;
			scan<<hostSwitch;
			bInputFile = false;
		}
		else
		{
			QString fileParsed;
			fileParsed = getFileName(host);

			if (fileParsed.isNull())
				KMsgBox::message (this,"KNmap","Cannot process this item \nsomething is wrong",KMsgBox::STOP);
			else
			{
				hostSwitch += fileParsed;
				scan<<hostSwitch;
			}
		}
	}	
	else
	{
   	scan<<host;

   	if(checkDoubles(host))
   		cboHost->insertItem(host);
 	}	

	if(scan.start(KProcess::NotifyOnExit,KProcess::AllOutput))
		toggleScan(false);
	else		
		KMsgBox::message (this,"KNmap","Could not start Nmap",KMsgBox::STOP);
	

}

void KNmap::slotCancel()
{
	scan.kill(SIGTERM);
	toggleScan(true);
}

void KNmap::slotQuit()
{
	saveConfig();

	// fixes a segfault problem
	// with KShellProcess
   delete this;
	kapp->quit();
}

void KNmap::slotFileOpen()
{

	QString fileName = QFileDialog::getOpenFileName (getenv("HOME"));

	if( !fileName.isEmpty())
	{
		QFile file(fileName);
		
		if (file.open(IO_ReadOnly))
		{
			
			QTextStream stream(&file);
			QString line;
			
			// @TODO check if nmap log
			while(!stream.eof())
			{
				line = stream.readLine();
				txtResult->append(line);
			}
			file.close();
		}	
	}
}

void KNmap::slotFileSave()
{
	QString fileName = QFileDialog::getSaveFileName (getenv("HOME"));

	if( !fileName.isEmpty())
	{
		QFile file(fileName);
		
		if (file.open(IO_WriteOnly))
		{
			QTextStream stream(&file);
			QString line(txtResult->text());
			stream<<line;
			file.close();
		}	
	}
}

void KNmap::slotInput()
{
	QString fileName = QFileDialog::getOpenFileName (getenv("HOME"));

	if( !fileName.isEmpty())
	{
		QFile file(fileName);
		
		if (file.exists())
		{
			fileForHost = fileName;
			fileName = "Input File - " + fileName;
			cboHost->insertItem(fileName,0);
			bInputFile = true;	
		}	
	}
}

void KNmap::slotOption()
{
	KMsgBox::message (this,"KNmap","Options not implemented yet",KMsgBox::INFORMATION);
}


void KNmap::saveConfig()
{
   QStrList hosts;
	
	for (int i=0;i<cboHost->count();i++)
	{
		QString str(cboHost->text(i));
		if(!str.isEmpty())	
			hosts.append(cboHost->text(i));
   }
   	
	kapp->getConfig()->writeEntry("hosts",hosts);
	kapp->getConfig()->writeEntry("winw",width());
 	kapp->getConfig()->writeEntry("winh",height());
   kapp->getConfig()->sync();
}

bool KNmap::checkDoubles(QString host)
{	
	for (int i=0;i<cboHost->count();i++)	
	{
		if(cboHost->text(i) == host)
			return false;
	}	
	return true;

}


