/***************************************************************************
                          knmap.h  -  description
                             -------------------
    begin                : Thu May 25 2000
    copyright            : (C) 2000 by Alexandre Sagala
    email                : rewind2@videotron.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNMAP_H
#define KNMAP_H

#include <qwidget.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qmultilinedit.h>

#include <ktmainwindow.h>
#include <kapp.h>
#include <kprocess.h>

#include <qlist.h>
#include <qlayout.h>

class KShellProcess;
class QGridLayout;

class KNmap : public KTMainWindow
{
   Q_OBJECT
public: 
	KNmap();
	~KNmap();

private:

	// variables
	bool bRoot;	
	bool bBounce;
	bool bRpc;
	bool bTcpScan;
	bool bInputFile;
	
	int scanId;
	int logId;
	
	QString fileForHost;
	
	KShellProcess scan;
	
	QPopupMenu *fileMenu;
	QPopupMenu *logMenu;
	QPopupMenu *optionMenu;
	QPopupMenu *helpMenu;

	// functions
	bool checkDoubles(QString);
	void disableNonRoot();
	void uncheckLogMenu();
	void rootCheck();
	void restoreConfig();
	void saveConfig();
	void toggleDecoys(bool);
	void toggleOthers(bool);
	void toggleScan(bool);
	
	QString getScanType();
	QString getLogType();
	QString getPingType();
	QString getFileName(QString);
	QStrList getOtherOptions();	
	
private slots:
	
	void commandCallback(int);
	void slotGroupScan(int);
	void slotBounce();
	void slotRpc();
	void slotLog(int);
	void slotFileOpen();
	void slotFileSave();
	void slotOption();
	void slotAdd();
	void slotDel();
	void slotScan();
	void slotCancel();
	void slotQuit();
	void slotInput();
	
	void slotScanStdout(KProcess*,char*,int);
	void slotScanStderr(KProcess*,char*,int);
	void slotScanExited(KProcess*);	
	
protected:

	void initGUI();
	void initMenuBar();
	void initConnect();

	QLabel *lblHost;
	QLabel *lblDecoy;
	QLabel *lblPort;
	QLabel *lblTiming;
	QLabel *lblPing;
	QLabel *lblResolve;
	
	QButtonGroup *groupScan;
	QButtonGroup *groupOptions;
	
	QRadioButton *optTcp;
	QRadioButton *optSyn;
	QRadioButton *optNull;
	QRadioButton *optFin;
	QRadioButton *optXmas;
	QRadioButton *optPingSweep;
	QRadioButton *optUdp;
	
	QComboBox *cboHost;
	QComboBox *cboPing;
	QComboBox *cboResolve;
	QComboBox *cboTiming;
	
	QLineEdit *txtPort;
	QLineEdit *txtDecoy;
	
	QListBox *lstDecoy;

	QCheckBox *chkRpc;
	QCheckBox *chkBounce;
	QCheckBox *chkRandom;
	QCheckBox *chkFastScan;
	QCheckBox *chkIdentd;
	QCheckBox *chkFragment;
	QCheckBox *chkOs;
	
	QMultiLineEdit *txtResult;
	
	QPushButton *cmdScan;
	QPushButton *cmdQuit;
	QPushButton *cmdCancel;
	QPushButton *cmdDel;
	QPushButton *cmdAdd;
	QPushButton *cmdInput;
		
	QGridLayout *glayout;
	QHBoxLayout *hlayoutTop;
	QHBoxLayout *hlayoutButtons;
	QHBoxLayout *hlayoutHost;
	QHBoxLayout *hlayoutMiddle;
};

#endif
